from bitfinex.client import Client

client = Client()

symbols = client.symbols()
print(symbols)

symbol = 'btcusd'

print(client.ticker(symbol))
print(client.today(symbol))
print(client.stats(symbol))

parameters = {'limit_asks': 2, 'limit_bids': 2}

print(client.lendbook('btc', parameters))
print(client.order_book(symbol, parameters))



#!/usr/bin/env python3
import os
import threading
import logging
from time import sleep
from bitfinex.websockets.client import WssClient

logging.basicConfig(
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    level=logging.DEBUG
)
LOGGER = logging.getLogger(__name__)


KEY = os.environ.get("API_KEY")
SECRET = os.environ.get("API_SECRET")


class MyWssTest():

    def __init__(self):
        self.mywss = WssClient(key=KEY, secret=SECRET)
        self.mywss2 = None
        # Tracks Websocket Connection
        self.connection_timer = None
        self.connection_timeout = 15
        self.keep_running = True
        self.wssactions = {
            'hb': self._heartbeat_handler
        }
        self.mywss.authenticate(self.cb_auth)
        self.mywss.start()
        sleep(5)
        self._start_timers()

    def cb_auth(self, message):
        LOGGER.info(f"cb_auth received {message}")
        if isinstance(message, list):
            msg_type = message[1]
            if msg_type in self.wssactions:
                self.wssactions[msg_type]()

    def _stop_timers(self):
        """Stops connection timers."""
        if self.connection_timer:
            self.connection_timer.cancel()
        LOGGER.info("_stop_timers(): Timers stopped.")

    def _start_timers(self):
        """Resets and starts timers for API data and connection."""
        LOGGER.info("_start_timers(): Resetting timers..")
        self._stop_timers()
        # Automatically reconnect if we didnt receive data
        self.connection_timer = threading.Timer(self.connection_timeout, self._connection_timed_out)
        self.connection_timer.start()

    def _connection_timed_out(self):
        """Issues a reconnection if the connection timed out.
        :return:
        """
        LOGGER.info("_connection_timed_out(): Fired! Issuing reconnect..")
        self.mywss.authenticate(self.cb_auth)

    def _heartbeat_handler(self):
        LOGGER.info("new hb received")
        LOGGER.info(f"nonce is {self.mywss._nonce()}")
        self._start_timers()

    def run(self):
        counter = 0
        while self.keep_running:
            sleep(1)
            counter += 1
            infomsg = (f'#run nr : {counter} || '
                       f'thread alive : {self.mywss.is_alive()} || '
                       f'nr of active  threads : {threading.active_count()}')
            print(infomsg)

            # example new order
            # if counter == 5:
            #     exch_order = self.mywss.new_order("EXCHANGE LIMIT", "tIOTUSD", "-1000", "30")
            #     print(f"new order is {exch_order}")

            # example update order
            # if counter == 7:
            #     update_info = {
            #         "id": "16666882406",
            #         "delta": "-400"
            #     }
            #     self.mywss.update_order(**update_info)

            # example use calculations
            if counter == 8:
                self.mywss.calc(["position_tIOTUSD"])

            # first reconnect test
            if counter == 20:
                self.stopit()

            # second reconnect test
            if counter == 50:
                self.stopit()

    def stopit(self):
        LOGGER.info("trying to stop")
        self.mywss.close()


print("###############START##############")
MYOBJ = MyWssTest()
print("###############END##############")

MYOBJ.run()



"""examples on how to use different methods from the rest library"""

from bitfinex import ClientV1 as Client1
from bitfinex import ClientV2 as Client2


if __name__ == "__main__":
    CLIENT = Client1('apiKey', 'apiSecret')

    """
    example on how to submit a rest request with multiple orders
    The following will make a list with 3 orders to buy 100 iota at 3 dollars,
    100 iota at 4 dollars and 100 iota at 5 dollars
    The list is sent to the method place_multiple_orders
    """
    ORDERS = []
    for price in range(3, 6):
        print(price)
        payload = {
            "symbol": 'IOTUSD',
            "amount": '100',
            "price": str(price),
            "exchange": 'bitfinex',
            "side": 'buy',
            "type": 'limit'
        }
        ORDERS.append(payload)

    APIRESPONSE = CLIENT.place_multiple_orders(ORDERS)
    print(APIRESPONSE)

    # REST API V2 examples
    BTFXCLIENT = Client2('apiKey', 'apiSecret')

    # REST PUBLIC ENDPOINTS
    BTFXCLIENT.platform_status()             # Paltform Status
    BTFXCLIENT.tickers(['tIOTUSD', 'fIOT'])  # list of tickers
    BTFXCLIENT.ticker('tIOTUSD')             # ticker
    BTFXCLIENT.trades('fIOT')                # public trades for symbol
    BTFXCLIENT.trades('tIOTUSD')             # public trades for symbol
    BTFXCLIENT.books('tIOTUSD')              # books for tIOTUSD with price precision 0
    BTFXCLIENT.books('tIOTUSD', "P1")        # books for tIOTUSD with price precision 1

    PARAMS = {
        'key': 'funding.size',
        'size': '1m',
        'symbol': 'fUSD',
        'section': 'hist',
        'sort': '0'
    }
    BTFXCLIENT.stats(**PARAMS)              # statistics

    PARAMS = {
        'key': 'credits.size',
        'size': '1m',
        'symbol': 'fUSD',
        'section': 'hist',
        'sort': '0'
    }
    BTFXCLIENT.stats(**PARAMS)              # statistics

    PARAMS = {
        'key': 'pos.size',
        'size': '1m',
        'symbol': 'tIOTUSD',
        'side': 'short',
        'section': 'hist',
        'sort': '0'
    }
    BTFXCLIENT.stats(**PARAMS)              # statistics

    PARAMS = {
        'key': 'credits.size.sym',
        'size': '1m',
        'symbol': 'fUSD',
        'symbol2': 'tBTCUSD',
        'section': 'hist',
        'sort': '0'
    }
    BTFXCLIENT.stats(**PARAMS)              # statistics

    BTFXCLIENT.candles("1m", "tBTCUSD", "hist")             # 1 minute candles
    BTFXCLIENT.candles("1m", "tBTCUSD", "hist", limit='1')  # 1 minute candles , limit to 1 candle

    # REST CALCULATION ENDPOINTS
    BTFXCLIENT.market_average_price(symbol="tBTCUSD", amount="100", period="1m")
    BTFXCLIENT.foreign_exchange_rate(ccy1="IOT", ccy2="USD")

    # REST AUTHENTICATED ENDPOINTS
    WB = BTFXCLIENT.wallets_balance()
    for wallet in WB:
        print(wallet)

    AO = BTFXCLIENT.active_orders("tIOTUSD")
    for order in AO:
        print(order)

    OH = BTFXCLIENT.orders_history("tIOTUSD")
    for order in OH:
        print(order)

    OT = BTFXCLIENT.order_trades("tIOTUSD", 14395751815)
    for trade in OT:
        print(trade)

    TRADES = BTFXCLIENT.trades_history('tIOTUSD', limit=10)
    TH = BTFXCLIENT.trades_history("tIOTUSD")
    for trade in TH:
        print(trade)

    BTFXCLIENT.active_positions()

    BTFXCLIENT.funding_offers()
    BTFXCLIENT.funding_offers("fIOT")
    BTFXCLIENT.funding_offers_history('fOMG')
    BTFXCLIENT.funding_loans('fOMG')
    BTFXCLIENT.funding_loans_history('fOMG')
    BTFXCLIENT.funding_credits('fUSD')
    BTFXCLIENT.funding_credits_history('fUSD')
    BTFXCLIENT.funding_trades('fUSD')

    BTFXCLIENT.margin_info()
    BTFXCLIENT.margin_info('base')
    BTFXCLIENT.margin_info('tIOTUSD')

    BTFXCLIENT.funding_info('fIOT')

    BTFXCLIENT.movements()

    BTFXCLIENT.performance()

    BTFXCLIENT.alert_list()
    BTFXCLIENT.alert_set('price', 'tIOTUSD', 3)
    BTFXCLIENT.alert_delete('tIOTUSD', 1)

    BTFXCLIENT.calc_available_balance('tIOTUSD', 1, 0.02, 'EXCHANGE')

    BTFXCLIENT.ledgers('IOT')
